package com.gemho.environment.controller;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import lombok.extern.slf4j.Slf4j;
import com.gemho.environment.constants.Constants;
import com.gemho.environment.constants.SensorToRegisterConstants;
import com.gemho.environment.utils.CRC16Util;
import com.gemho.environment.utils.StringUtil;
import com.gemho.environment.utils.RedisUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * @Author: liujing
 * @Date: 2019/10/26 10:04
 * @Version: 1.0
 */

@Component
@Slf4j
public class ServerToRelayService {

    @Resource
    RedisUtils redisUtil;

    /**
     * 发送传感器数据到控制器
     * @param sensorid 监测设备ID
     * @param item 检测项
     * @param vale 监测数据
     */
    public void sendDataToRelay(String sensorid, String item, Double vale) {
        log.debug("执行>>sendDataToRelay<< :监测设备ID{}, item:{}, value:{}",sensorid,item,vale);
            try {
                // deviceId: 控制器
                String channelID = redisUtil.hget(Constants.REDIS_DEVICE_CHANNEL_KEY, "deviceId").toString();
                //获取item对应的寄存器地址
                JSONObject jsonObject = JSONUtil.parseObj(SensorToRegisterConstants.SENSOR_TO_REGISTER);
                String register = String.valueOf(jsonObject.get("TMP2")); //寄存器地址
                //数据转成十六进制码表示
                String hexVale = Integer.toHexString(vale.intValue());
                if (hexVale.length() < 8) {
                    hexVale = StringUtils.leftPad(hexVale, 8, "0").toUpperCase();
                }
                log.debug("hexVale:{}", hexVale);
                //拼接指令
                String command = "deviceId"+"80"+register+hexVale;
                byte[] resByte = StringUtil.hexString2Bytes(command);
                String resCRC = Integer.toHexString(CRC16Util.calcCrc16(resByte)).toUpperCase();
                if (resCRC.length() < 4) {
                    resCRC = StringUtils.leftPad(resCRC, 4, "0").toUpperCase();
                }
                command = command+resCRC;
                Integer dataLength = resByte.length+2;
                command = "C33C"+Integer.toHexString(dataLength)+command;
                String finalCommand = command.toUpperCase();
                // deviceId:  控制器
                log.debug("向{}控制器发送设备{}下{}传感器的指令:{}","deviceId",sensorid,item,finalCommand);
                ServerHandler.channelGroup.forEach(ch -> {
                    // channelId 查看链接通道
                    if (ch.id().toString().equals(channelID)) {
                        ByteBuf bufff = Unpooled.buffer();
                        //  bufff.clear();
                        bufff.writeBytes(StringUtil.hexString2Bytes(finalCommand));
                        ch.writeAndFlush(bufff);
                    }
                });
            }catch (Exception e) {
                log.error(e.getMessage());
            }
    }

    public void sendDataTo4GMonitor(String sensorid, String data) {
        log.debug("执行>>sendDataTo4GMonitor<< sensorid:{}, data:{}",sensorid,data);
        //设备关联的控制器
        // deviceId : 控制器
        log.debug("redis是否存在LED设备:"+"deviceId"+":"+redisUtil.hHasKey(Constants.REDIS_DEVICE_CHANNEL_KEY, "deviceId"));
        if (redisUtil.hHasKey(Constants.REDIS_DEVICE_CHANNEL_KEY, "deviceId") == true) {
            try{
                //获取通道ID
                String channelID = redisUtil.hget(Constants.REDIS_DEVICE_CHANNEL_KEY, "deviceId").toString();
                ServerHandler.channelGroup.forEach(ch -> {
                    if (ch.id().toString().equals(channelID)) {
                        ch.writeAndFlush(data);
                    }
                });
            }catch (Exception e) {
                log.error(e.getMessage());
            }
        }
    }
}
