package com.gemho.environment.service;


import com.gemho.environment.model.Relay;
import com.gemho.environment.model.RelayDto;
import com.gemho.environment.model.RelayQueryCriteria;
import org.springframework.data.domain.Pageable;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
* @author 靖小侠
* @date 2020-05-07
*/
public interface RelayService {

    /**
    * 查询数据分页
    * @param criteria 条件
    * @param pageable 分页参数
    * @return Map<String,Object>
    */
    Map<String,Object> queryAll(RelayQueryCriteria criteria, Pageable pageable);

    /**
    * 查询所有数据不分页
    * @param criteria 条件参数
    * @return List<RelayDto>
    */
    List<RelayDto> queryAll(RelayQueryCriteria criteria);

    /**
     * 根据ID查询
     * @param id ID
     * @return RelayDto
     */
    RelayDto findById(Long id);

    /**
    * 创建
    * @param resources /
    * @return RelayDto
    */
    RelayDto create(Relay resources);

    /**
    * 编辑
    * @param resources /
    */
    void update(Relay resources);

    /**
    * 多选删除
    * @param ids /
    */
    void deleteAll(Long[] ids);

    /**
    * 导出数据
    * @param all 待导出的数据
    * @param response /
    * @throws IOException /
    */
    void download(List<RelayDto> all, HttpServletResponse response) throws IOException;
    /**
     * 通过imei查询设备
     * @param imei
     * @return
     */
    RelayDto findByImei(String imei);
    /**
     * 通过deviceid查询设备
     * @param deviceid
     * @return
     */
    RelayDto findByDeviceid(String deviceid);

    Map<String,Object> queryAllByUserid(Long userid, RelayQueryCriteria criteria, Pageable pageable);

    void unbind(String relayid);

    List<RelayDto> queryAllRelayByUseridNoPage(Long userid);

    String sendCommand(String relayId, String command);

    /**
     * 更新设备在线时间
     * @param deviceId
     */
    public void updateRelayOnlinetime(String deviceId);

    /**
     * 更新设备工作模式
     * @param deviceId
     * @param workmode
     */
    public void updateRelayWorkMode(String deviceId, String workmode);

    Map<String,Object> queryRelayAndSensorByUserid(Long userid, RelayQueryCriteria criteria, Pageable pageable);

    List<RelayDto> findRelayByMonitor(Long userid, String id);

    int queryOnlineRelayByUser(Long userid);
    int queryNotOnlineRelayByUser(Long userid);
    int queryOnlineRelayBySuper();
    int queryNotOnlineRelayBySuper();
}